#pragma once
#include "Extensions.h"
#include "ScriptNode.h"

#include "BasicAtl.h"

class ATL_NO_VTABLE ScriptManager : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public ISupportErrorInfoImpl<&IID_IVirtualFileSystem>,
	public IDispatchImpl<IVirtualFileSystem, &IID_IVirtualFileSystem, &__uuidof(__ww10_com), 0xffff, 0xffff>
{
	_IBasicIdeCtl* basicIdeCtl_;
	std::wstring root_;
	CBasicNoUIObj basicNoUIObj_;
	CComPtr<CExtensions> extensions_;
	bool fast_;

	std::list<ScriptNode*> loaded_script_nodes_;
	ScriptNode* active_script_node_;

public:
	using script_loaded_status_changed_callback = std::function<void(const std::wstring&, bool)>;
private:
	script_loaded_status_changed_callback script_loaded_status_changed_;

public:
	ScriptManager() : basicIdeCtl_(0), fast_(false), active_script_node_(0) { }
	~ScriptManager() { }

public:
	HRESULT FinalConstruct() { return S_OK; }
	void FinalRelease() { extensions_.Release(); }

	HRESULT Attach(_IBasicIdeCtlPtr basicIdeCtl, const wchar_t* root, script_loaded_status_changed_callback script_loaded_status_changed);
	void Disconnect();
	void EnterDesignMode(VARIANT_BOOL* pCancel);
	HRESULT EvaluateScript(ScriptNode* script_node, VARIANT& result, const VARIANT* default_value);
	void EvaluateScripts(const std::list<std::wstring>& script_names, std::list<variant_t>& results, const VARIANT* default_value);
	void Fast(bool fast) { fast_ = fast; }
	HRESULT LoadScript(const wchar_t* script_name, ScriptNode*& script_node);
	bool Shutdown();
	void SynchronizeIde(BSTR* Param, long Id);
	void SynchronizeNoUI(BSTR* Param, long Id);

	DECLARE_PROTECT_FINAL_CONSTRUCT()

	BEGIN_COM_MAP(ScriptManager)
		COM_INTERFACE_ENTRY(ISupportErrorInfo)
		COM_INTERFACE_ENTRY(IVirtualFileSystem)
		COM_INTERFACE_ENTRY(IDispatch)
	END_COM_MAP()

public:
	virtual HRESULT __stdcall GetCaption(BSTR Name, BSTR* pretval) override;
	virtual HRESULT __stdcall GetTimeStamp(BSTR Name, DATE * pretval ) override;
	virtual HRESULT __stdcall Combine(BSTR BaseName, BSTR Name, BSTR * pretval ) override;
	virtual HRESULT __stdcall Exists(BSTR Name, VARIANT_BOOL * pretval) override;
	virtual HRESULT __stdcall Read(BSTR Name, BSTR * pretval) override;
	virtual HRESULT __stdcall Write(BSTR Name, BSTR text) override;
	virtual HRESULT __stdcall Delete(BSTR Name) override;
};

